@@ -205,11 +205,12 @@ def assert_close(actual, expected, atol=1e-6, rtol=1e-6):
             elif atol is not None:
                 assert diff.max() < atol, msg
     elif isinstance(actual, numbers.Number):
-        diff = abs(actual - expected)
-        if rtol is not None:
-            assert diff < (atol + abs(expected)) * rtol, msg
-        elif atol is not None:
-            assert diff < atol, msg
+        if actual != expected:
+            diff = abs(actual - expected)
+            if rtol is not None:
+                assert diff < (atol + abs(expected)) * rtol, msg
+            elif atol is not None:
+                assert diff < atol, msg
     elif isinstance(actual, dict):
         assert isinstance(expected, dict)
         assert set(actual) == set(expected)
@@ -238,9 +239,17 @@ def check_funsor(x, inputs, output, data=None):
         else:
             x_data = x.align(tuple(inputs)).data
         if inputs or output.shape:
-            assert (x_data == data).all()
+            if get_backend() == "jax":
+                # JAX has numerical errors for reducing ops.
+                assert_close(x_data, data)
+            else:
+                assert (x_data == data).all()
         else:
-            assert x_data == data
+            if get_backend() == "jax":
+                # JAX has numerical errors for reducing ops.
+                assert_close(x_data, data)
+            else:
+                assert x_data == data
 
 
 def xfail_param(*args, **kwargs):
