@@ -198,7 +198,7 @@ def _cholesky_inverse(x):
 def _triangular_inv(x, upper=False):
     if x.size(-1) == 1:
         return x.reciprocal()
-    return torch.eye(x.size(-1)).triangular_solve(x, upper=upper).solution
+    return torch.linalg.solve_triangular(x, torch.eye(x.size(-1)), upper=upper)
 
 
 @ops.detach.register(torch.Tensor)
@@ -334,7 +334,7 @@ def _permute(x, dims):
 
 @ops.pow.register(numbers.Number, torch.Tensor)
 def _pow(x, y):
-    result = x ** y
+    result = x**y
     # work around shape bug https://github.com/pytorch/pytorch/issues/16685
     return result.reshape(y.shape)
 
@@ -342,12 +342,12 @@ def _pow(x, y):
 @ops.pow.register(torch.Tensor, numbers.Number)
 @ops.pow.register(torch.Tensor, torch.Tensor)
 def _pow(x, y):
-    return x ** y
+    return x**y
 
 
 @ops.reciprocal.register(torch.Tensor)
 def _reciprocal(x):
-    if x.dtype in (torch.complex32, torch.complex64):
+    if torch.is_complex(x):
         return x.reciprocal()
     result = x.reciprocal().clamp(max=torch.finfo(x.dtype).max)
     return result
@@ -393,4 +393,7 @@ ops.stack.register(typing.Tuple[torch.Tensor, ...])(torch.stack)
 def _triangular_solve(x, y, upper=False, transpose=False):
     if y.size(-1) == 1:
         return x / y
-    return x.triangular_solve(y, upper, transpose).solution
+    if transpose:
+        y = y.transpose(-1, -2)
+        upper = not upper
+    return torch.linalg.solve_triangular(y, x, upper=upper)
